@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
  * Copyright (C) 2003 Imendio AB
  *
@@ -13,23 +13,27 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this program; if not, see <https://www.gnu.org/licenses>
+ */
+
+/**
+ * SECTION:lm-error
+ * @Title: LmError
+ * @Short_description: Error reporting.
  */
 
 #include <config.h>
-#include <glib/gerror.h>
+#include <glib.h>
 #include "lm-error.h"
 
 GQuark
 lm_error_quark (void)
 {
-	static GQuark q = 0;
+    static GQuark q = 0;
 
-	if (q == 0) {
-		q = g_quark_from_static_string ("lm-error-quark");
-	}
-	
-	return q;
+    if (q == 0) {
+        q = g_quark_from_static_string ("lm-error-quark");
+    }
+
+    return q;
 }
