@@ -1,6 +1,6 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- * Copyright (C) 2003 Imendio HB
+ * Copyright (C) 2003 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License as
@@ -13,9 +13,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this program; if not, see <https://www.gnu.org/licenses>
  */
 
 #ifndef __LM_SHA_H__
@@ -23,6 +21,6 @@
 
 #include <glib.h>
 
-const gchar *     lm_sha_hash    (const gchar *str);
+gchar *     lm_sha_hash    (const gchar *str);
 
 #endif /* __LM_SHA_H__ */
