@@ -13,9 +13,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this program; if not, see <https://www.gnu.org/licenses>
  */
 
 #include <config.h>
@@ -36,8 +34,7 @@
 #ifdef HAVE_GNUTLS
 
 #include <gnutls/x509.h>
-
-#define CA_PEM_FILE "/etc/ssl/certs/ca-certificates.crt"
+#include <gnutls/crypto.h>
 
 struct _LmSSL {
     LmSSLBase base;
@@ -113,6 +110,8 @@ ssl_verify_certificate (LmSSL *ssl, const gchar *server)
     if (gnutls_certificate_type_get (ssl->gnutls_session) == GNUTLS_CRT_X509) {
         const gnutls_datum_t* cert_list;
         guint cert_list_size;
+        gnutls_digest_algorithm_t digest = GNUTLS_DIG_SHA256;
+        guchar digest_bin[LM_FINGERPRINT_LENGTH];
         size_t digest_size;
         gnutls_x509_crt_t cert;
 
@@ -143,15 +142,15 @@ ssl_verify_certificate (LmSSL *ssl, const gchar *server)
 
         gnutls_x509_crt_deinit (cert);
 
-        digest_size = sizeof (base->fingerprint);
+        digest_size = gnutls_hash_get_len(digest);
+        g_assert(digest_size < sizeof(digest_bin));
 
-        if (gnutls_fingerprint (GNUTLS_DIG_MD5, &cert_list[0],
-                                base->fingerprint,
+        if (gnutls_fingerprint (digest,
+                                &cert_list[0],
+                                digest_bin,
                                 &digest_size) >= 0) {
-            if (base->expected_fingerprint &&
-                memcmp (base->expected_fingerprint,
-                        base->fingerprint,
-                        digest_size) &&
+            _lm_ssl_base_set_fingerprint(base, digest_bin, digest_size);
+            if (_lm_ssl_base_check_fingerprint(base) != 0 &&
                 base->func (ssl,
                             LM_SSL_STATUS_CERT_FINGERPRINT_MISMATCH,
                             base->func_data) != LM_SSL_RESPONSE_CONTINUE) {
@@ -191,9 +190,6 @@ _lm_ssl_initialize (LmSSL *ssl)
 {
     gnutls_global_init ();
     gnutls_certificate_allocate_credentials (&ssl->gnutls_xcred);
-    gnutls_certificate_set_x509_trust_file(ssl->gnutls_xcred,
-                                           CA_PEM_FILE,
-                                           GNUTLS_X509_FMT_PEM);
 }
 
 gboolean
@@ -272,12 +268,14 @@ _lm_ssl_begin (LmSSL *ssl, gint fd, const gchar *server, GError **error)
     base = LM_SSL_BASE(ssl);
     gnutls_init (&ssl->gnutls_session, GNUTLS_CLIENT);
     if (base->cipher_list) {
-      gnutls_priority_set_direct (ssl->gnutls_session, base->cipher_list, NULL);
+        gnutls_priority_set_direct (ssl->gnutls_session, base->cipher_list, NULL);
     } else {
-      gnutls_priority_set_direct (ssl->gnutls_session, "NORMAL", NULL);
+        gnutls_priority_set_direct (ssl->gnutls_session, "NORMAL", NULL);
     }
     if (base->ca_path) {
-      _lm_ssl_set_ca(ssl, base->ca_path);
+        _lm_ssl_set_ca(ssl, base->ca_path);
+    } else {
+        gnutls_certificate_set_x509_system_trust(ssl->gnutls_xcred);
     }
     gnutls_credentials_set (ssl->gnutls_session,
                             GNUTLS_CRD_CERTIFICATE,
@@ -286,7 +284,9 @@ _lm_ssl_begin (LmSSL *ssl, gint fd, const gchar *server, GError **error)
     gnutls_transport_set_ptr (ssl->gnutls_session,
                               (gnutls_transport_ptr_t)(glong) fd);
 
-    ret = gnutls_handshake (ssl->gnutls_session);
+    do {
+        ret = gnutls_handshake(ssl->gnutls_session);
+    } while (GNUTLS_E_AGAIN == ret || GNUTLS_E_INTERRUPTED == ret);
 
     if (ret >= 0) {
         auth_ok = ssl_verify_certificate (ssl, server);
@@ -313,9 +313,11 @@ _lm_ssl_begin (LmSSL *ssl, gint fd, const gchar *server, GError **error)
                 gnutls_cipher_suite_get_name(gnutls_kx_get(ssl->gnutls_session),
                                              gnutls_cipher_get(ssl->gnutls_session),
                                              gnutls_mac_get(ssl->gnutls_session)));
+#if GNUTLS_VERSION_NUMBER < 0x030600
     lm_verbose ("GNUTLS negotiated compression: %s",
                 gnutls_compression_get_name (gnutls_compression_get
                                              (ssl->gnutls_session)));
+#endif
 
     ssl->started = TRUE;
 
